@@ -1,20 +1,17 @@
-/*============================================================================
-  CMake - Cross Platform Makefile Generator
-  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
-
-  Distributed under the OSI-approved BSD License (the "License");
-  see accompanying file Copyright.txt for details.
-
-  This software is distributed WITHOUT ANY WARRANTY; without even the
-  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-  See the License for more information.
-============================================================================*/
+/* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
+   file Copyright.txt or https://cmake.org/licensing for details.  */
 #include "cmCTestStartCommand.h"
 
 #include "cmCTest.h"
 #include "cmCTestVC.h"
 #include "cmGeneratedFileStream.h"
-#include "cmGlobalGenerator.h"
+#include "cmMakefile.h"
+#include "cmSystemTools.h"
+
+#include <sstream>
+#include <stddef.h>
+
+class cmExecutionStatus;
 
 cmCTestStartCommand::cmCTestStartCommand()
 {
@@ -23,49 +20,49 @@ cmCTestStartCommand::cmCTestStartCommand()
 }
 
 bool cmCTestStartCommand::InitialPass(std::vector<std::string> const& args,
-                                      cmExecutionStatus&)
+                                      cmExecutionStatus& /*unused*/)
 {
-  if (args.size() < 1) {
+  if (args.empty()) {
     this->SetError("called with incorrect number of arguments");
     return false;
   }
 
   size_t cnt = 0;
-  const char* smodel = args[cnt].c_str();
-  const char* src_dir = 0;
-  const char* bld_dir = 0;
+  const char* smodel = nullptr;
+  const char* src_dir = nullptr;
+  const char* bld_dir = nullptr;
 
-  cnt++;
-
-  this->CTest->SetSpecificTrack(0);
-  if (cnt < args.size() - 1) {
+  while (cnt < args.size()) {
     if (args[cnt] == "TRACK") {
       cnt++;
+      if (cnt >= args.size() || args[cnt] == "APPEND" ||
+          args[cnt] == "QUIET") {
+        this->SetError("TRACK argument missing track name");
+        return false;
+      }
       this->CTest->SetSpecificTrack(args[cnt].c_str());
       cnt++;
-    }
-  }
-
-  if (cnt < args.size()) {
-    if (args[cnt] == "APPEND") {
+    } else if (args[cnt] == "APPEND") {
       cnt++;
       this->CreateNewTag = false;
-    }
-  }
-  if (cnt < args.size()) {
-    if (args[cnt] == "QUIET") {
+    } else if (args[cnt] == "QUIET") {
       cnt++;
       this->Quiet = true;
+    } else if (!smodel) {
+      smodel = args[cnt].c_str();
+      cnt++;
+    } else if (!src_dir) {
+      src_dir = args[cnt].c_str();
+      cnt++;
+    } else if (!bld_dir) {
+      bld_dir = args[cnt].c_str();
+      cnt++;
+    } else {
+      this->SetError("Too many arguments");
+      return false;
     }
   }
 
-  if (cnt < args.size()) {
-    src_dir = args[cnt].c_str();
-    cnt++;
-    if (cnt < args.size()) {
-      bld_dir = args[cnt].c_str();
-    }
-  }
   if (!src_dir) {
     src_dir = this->Makefile->GetDefinition("CTEST_SOURCE_DIRECTORY");
   }
@@ -82,6 +79,11 @@ bool cmCTestStartCommand::InitialPass(std::vector<std::string> const& args,
                    "as an argument or set CTEST_BINARY_DIRECTORY");
     return false;
   }
+  if (!smodel && this->CreateNewTag) {
+    this->SetError("no test model specified and APPEND not specified. Specify "
+                   "either a test model or the APPEND argument");
+    return false;
+  }
 
   cmSystemTools::AddKeepPath(src_dir);
   cmSystemTools::AddKeepPath(bld_dir);
@@ -95,11 +97,22 @@ bool cmCTestStartCommand::InitialPass(std::vector<std::string> const& args,
   this->CTest->SetCTestConfiguration("BuildDirectory", binaryDir.c_str(),
                                      this->Quiet);
 
-  cmCTestOptionalLog(this->CTest, HANDLER_OUTPUT, "Run dashboard with model "
-                       << smodel << std::endl
-                       << "   Source directory: " << src_dir << std::endl
-                       << "   Build directory: " << bld_dir << std::endl,
-                     this->Quiet);
+  if (smodel) {
+    cmCTestOptionalLog(this->CTest, HANDLER_OUTPUT,
+                       "Run dashboard with model "
+                         << smodel << std::endl
+                         << "   Source directory: " << src_dir << std::endl
+                         << "   Build directory: " << bld_dir << std::endl,
+                       this->Quiet);
+  } else {
+    cmCTestOptionalLog(this->CTest, HANDLER_OUTPUT,
+                       "Run dashboard with "
+                       "to-be-determined model"
+                         << std::endl
+                         << "   Source directory: " << src_dir << std::endl
+                         << "   Build directory: " << bld_dir << std::endl,
+                       this->Quiet);
+  }
   const char* track = this->CTest->GetSpecificTrack();
   if (track) {
     cmCTestOptionalLog(this->CTest, HANDLER_OUTPUT,
@@ -129,9 +142,14 @@ bool cmCTestStartCommand::InitialPass(std::vector<std::string> const& args,
     return false;
   }
 
-  this->Makefile->AddDefinition("CTEST_RUN_CURRENT_SCRIPT", "OFF");
+  this->CTest->SetRunCurrentScript(false);
   this->CTest->SetSuppressUpdatingCTestConfiguration(true);
-  int model = this->CTest->GetTestModelFromString(smodel);
+  int model;
+  if (smodel) {
+    model = this->CTest->GetTestModelFromString(smodel);
+  } else {
+    model = cmCTest::UNKNOWN;
+  }
   this->CTest->SetTestModel(model);
   this->CTest->SetProduceXML(true);
 
